# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation,inconsistent-return-statements
"""
Submit data to the Polarion XUnit/TestCase Importer.
"""

from __future__ import absolute_import, unicode_literals

import logging
import os

from dump2polarion import configuration, utils
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.verify import verify_submit


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def _get_xml_root(xml_root, xml_str, xml_file):
    if xml_root is not None:
        return xml_root
    if xml_str:
        return utils.get_xml_root_from_str(xml_str)
    if xml_file:
        return utils.get_xml_root(xml_file)
    raise Dump2PolarionException("Failed to submit to Polarion - no data supplied")


def _get_submit_target(xml_root, config):
    if xml_root.tag == 'testcases':
        target = config.get('testcase_taget')
    elif xml_root.tag == 'testsuites':
        target = config.get('xunit_target')
    else:
        raise Dump2PolarionException(
            "Failed to submit to Polarion - submit target not found")
    return target


def _get_queue_url(xml_root, config):
    if xml_root.tag == 'testcases':
        target = config.get('testcase_queue')
    elif xml_root.tag == 'testsuites':
        target = config.get('xunit_queue')
    else:
        return
    return target


def response2dict(response):
    """Returns dict of the response."""
    try:
        return response.json()
    # pylint: disable=broad-except
    except Exception:
        return


def get_job_ids(response):
    """Returns job IDs of the import."""
    if not isinstance(response, dict):
        response = response2dict(response)
    if response:
        return response['files']['results.xml']['job-ids']


def get_error_message(response):
    """Returns job IDs of the import."""
    if not isinstance(response, dict):
        response = response2dict(response)
    if response:
        return response['files']['results.xml'].get('error-message')


def validate_response(response, submit_target):
    """Checks that the response is valid and import succeeded."""
    if response is None:
        logger.error('Failed to submit to {}'.format(submit_target))
        return False

    if not response:
        logger.error('HTTP status {}: failed to submit to {}'.format(
            response.status_code, submit_target))
        return False

    parsed_response = response2dict(response)

    if not parsed_response:
        logger.error('Submit to {} failed, invalid response received'.format(submit_target))
        return False

    error_message = get_error_message(parsed_response)
    if error_message:
        logger.error('Submit to {} failed with error'.format(submit_target))
        logger.debug('Error message: {}'.format(error_message))
        return False

    job_ids = get_job_ids(parsed_response)
    if job_ids == [0]:
        logger.error('Submit to {} failed to get job id'.format(submit_target))
        return False

    logger.info('Results received by the Importer (HTTP status {})'.format(
        response.status_code))
    logger.info('Job IDs: {}'.format(job_ids))

    return response


def _get_credentials(config, **kwargs):
    login = kwargs.get('user') or config.get('username') or os.environ.get('POLARION_USERNAME')
    pwd = kwargs.get('password') or config.get('password') or os.environ.get('POLARION_PASSWORD')

    if not all([login, pwd]):
        raise Dump2PolarionException("Failed to submit to Polarion - missing credentials")

    return (login, pwd)


def submit(xml_str=None, xml_file=None, xml_root=None, config=None, session=None, **kwargs):
    """Submits data to the Polarion Importer."""
    try:
        config = config or configuration.get_config()
        xml_root = _get_xml_root(xml_root, xml_str, xml_file)
        credentials = _get_credentials(config, **kwargs)
        submit_target = _get_submit_target(xml_root, config)
        utils.xunit_fill_testrun_id(xml_root, kwargs.get('testrun_id'))
        xml_input = utils.etree_to_string(xml_root)
        session = session or utils.get_session(credentials, config)
    except Dump2PolarionException as err:
        logger.error(err)
        return

    logger.info("Submitting data to {}".format(submit_target))
    files = {'file': ('results.xml', xml_input)}
    try:
        response = session.post(submit_target, files=files)
    # pylint: disable=broad-except
    except Exception as err:
        logger.error(err)
        response = None

    return validate_response(response, submit_target)


def submit_and_verify(
        xml_str=None, xml_file=None, xml_root=None, config=None, session=None, **kwargs):
    """Submits data to the Polarion Importer and checks that it was imported."""
    try:
        config = config or configuration.get_config()
        xml_root = _get_xml_root(xml_root, xml_str, xml_file)
        credentials = _get_credentials(config, **kwargs)
        queue_url = _get_queue_url(xml_root, config)
        session = session or utils.get_session(credentials, config)
    except Dump2PolarionException as err:
        logger.error(err)
        return

    response = submit(xml_root=xml_root, config=config, session=session, **kwargs)
    if not response:
        return False

    job_ids = get_job_ids(response)
    if not kwargs.get('no_verify') and job_ids:
        response = verify_submit(
            session,
            queue_url,
            job_ids,
            timeout=kwargs.get('verify_timeout'),
            log_file=kwargs.get('log_file')
        )

    return bool(response)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,no-self-use,too-few-public-methods,protected-access


from __future__ import unicode_literals

import os

from mock import patch
from tests import conf

from dump2polarion import utils, submit


# pylint: disable=unused-argument
class DummySession(object):
    def __init__(self, method):
        self._method = method

    def get(self, *args, **kwargs):
        return self._method()

    def post(self, *args, **kwargs):
        return self._method()


class DummyResponse(object):
    def __init__(self, response=None):
        self.status_code = 200
        self.response = response or {}

    def __len__(self):
        return 1

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response


class TestSubmit(object):
    def test_missing_input(self, config_prop, captured_log):
        submit.submit('', config=config_prop)
        assert 'no data supplied' in captured_log.getvalue()

    def test_missing_credentials(self, config_prop, captured_log):
        submit.submit('<foo/>', config=config_prop)
        assert 'missing credentials' in captured_log.getvalue()

    def test_missing_target(self, config_prop, captured_log):
        submit.submit('<foo/>', config=config_prop, user='john', password='123')
        assert 'submit target not found' in captured_log.getvalue()

    def test_missing_testrun_id(self, config_prop, captured_log):
        submit.submit(
            '<testsuites><properties></properties></testsuites>',
            config=config_prop,
            user='john',
            password='123',
            session='foo',
        )
        assert 'missing testrun id' in captured_log.getvalue()

    def test_fill_testrun_submit(self, config_prop):
        fname = 'properties.xml'
        submit.submit(
            xml_file=os.path.join(conf.DATA_PATH, fname),
            testrun_id='5_8_0_17',
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(DummyResponse),
        )

    def test_get_testcases_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str('<testcases/>')
        response = submit._get_submit_target(xml_root, config_prop)
        assert 'testcase' in response

    def test_get_testsuites_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str('<testsuites/>')
        response = submit._get_submit_target(xml_root, config_prop)
        assert 'xunit' in response

    def test_file_testsuites_failure(self, config_prop, captured_log):
        class Response(object):
            def __init__(self):
                self.status_code = 404

            def __len__(self):
                return 0

        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        response = submit.submit(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(Response)
        )
        assert not response
        assert 'HTTP status 404: failed to submit' in captured_log.getvalue()

    def test_file_testsuites_none(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        response = submit.submit(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(lambda: None)
        )
        assert not response
        assert 'Failed to submit' in captured_log.getvalue()

    def test_file_testsuites_exception(self, config_prop, captured_log):
        def _raise():
            raise Exception('request failed')

        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        response = submit.submit(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(_raise)
        )
        assert not response
        logged_data = captured_log.getvalue()
        assert 'request failed' in logged_data
        assert 'Failed to submit' in logged_data

    def test_file_testsuites_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        response = submit.submit(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(
                lambda: DummyResponse({'files': {'results.xml': {'job-ids': [1, 2]}}}))
        )
        assert response
        logged_data = captured_log.getvalue()
        assert 'Results received' in logged_data
        assert 'Job IDs' in logged_data

    def test_file_testcases_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'testcases.xml')
        response = submit.submit(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(
                lambda: DummyResponse({'files': {'results.xml': {'job-ids': [1, 2]}}}))
        )
        assert response
        logged_data = captured_log.getvalue()
        assert 'Results received' in logged_data
        assert 'Job ID' in logged_data


class TestSubmitAndVerify(object):
    def test_missing_input(self, config_prop, captured_log):
        submit.submit_and_verify('', config=config_prop)
        assert 'no data supplied' in captured_log.getvalue()

    def test_no_verify(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user='john',
            password='123',
            session=DummySession(
                lambda: DummyResponse({'files': {'results.xml': {'job-ids': [1, 2]}}})),
            no_verify=True,
        )
        assert response
        assert 'Results received' in captured_log.getvalue()

    def test_verify_failed(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('dump2polarion.verify.QueueSearch') as mock:
            instance = mock.return_value
            instance.verify_submit.return_value = False
            response = submit.submit_and_verify(
                xml_file=input_file,
                config=config_prop,
                user='john',
                password='123',
                session=DummySession(
                    lambda: DummyResponse({'files': {'results.xml': {'job-ids': [1, 2]}}}))
            )
        assert not response
        assert 'Results received' in captured_log.getvalue()

    def test_verify_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('dump2polarion.verify.QueueSearch') as mock:
            instance = mock.return_value
            instance.verify_submit.return_value = True
            response = submit.submit_and_verify(
                xml_file=input_file,
                config=config_prop,
                user='john',
                password='123',
                session=DummySession(
                    lambda: DummyResponse({'files': {'results.xml': {'job-ids': [1, 2]}}}))
            )
        assert response
        assert 'Results received' in captured_log.getvalue()



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os

import pytest

from dump2polarion import configuration
from dump2polarion.exceptions import Dump2PolarionException
from tests import conf


class TestConfiguration(object):
    def test_nonexistant(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config('nonexistant')
        assert "Cannot open config file 'nonexistant'" in str(excinfo.value)

    def test_default(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config()
        assert 'Failed to find following keys in config file' in str(excinfo.value)

    def test_user(self, config_e2e):
        cfg = configuration.get_config(config_e2e)
        assert cfg['xunit_import_properties']['polarion-dry-run'] is False
        assert cfg['username'] == 'user1'
        assert cfg['xunit_import_properties']['polarion-project-id'] == 'RHCF3'

    @pytest.mark.parametrize('config', ['dump2polarion_legacy.yaml', 'dump2polarion.yaml'])
    def test_populate(self, config):
        conf_file = os.path.join(conf.DATA_PATH, config)
        cfg = configuration.get_config(conf_file)
        polarion_url = cfg.get('polarion_url')
        for key in configuration.URLS:
            assert key in cfg
            if polarion_url and cfg.get(key):
                assert polarion_url in cfg[key]

    def test_check_config(self):
        cfg = {}
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration._check_config(cfg)
        assert 'Failed to find following keys in config file' in str(excinfo.value)



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Verifies that data were updated in Polarion.
"""

from __future__ import absolute_import, unicode_literals

import logging
import os
import time

import requests


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


_DEFAULT_TIMEOUT = 600
_DEFAULT_DELAY = 10

_NOT_FINISHED_STATUSES = ('ready', 'running')


class QueueSearch(object):
    """Search for jobs in the completed jobs queue."""

    def __init__(self, session, queue_url):
        self.session = session
        self.queue_url = queue_url
        self.skip = False
        self._check_setup()

    def _check_setup(self):
        """Checks that all the data that is needed for submit verification is available."""
        if not self.queue_url:
            logger.error(
                'The queue url is not configured, skipping submit verification')
            self.skip = True
            return

        if not self.session:
            logger.error('Missing requests session, skipping submit verification')
            self.skip = True
            return

    # pylint:disable=inconsistent-return-statements
    def download_queue(self, job_ids):
        """Downloads data of completed jobs."""
        if self.skip:
            return

        url = '{0}?jobtype=completed&jobIds={1}'.format(
            self.queue_url, ','.join(str(x) for x in job_ids))
        try:
            response = self.session.get(
                url,
                headers={'Accept': 'application/json'}
            )
            if response:
                response = response.json()
            else:
                response = None
        # pylint: disable=broad-except
        except Exception as err:
            logger.error(err)
            response = None

        return response

    # pylint:disable=inconsistent-return-statements
    def find_jobs(self, job_ids):
        """Finds the jobs in the completed job queue."""
        if self.skip:
            return

        json_data = self.download_queue(job_ids)
        if not json_data:
            return

        jobs = json_data['jobs']
        matched_jobs = []
        for job in jobs:
            if (job.get('id') in job_ids and
                    job.get('status', '').lower() not in _NOT_FINISHED_STATUSES):
                matched_jobs.append(job)

        return matched_jobs

    def wait_for_jobs(self, job_ids, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY):
        """Waits until the jobs appears in the completed job queue."""
        if self.skip:
            return

        logger.debug(
            'Waiting up to {} sec for completion of the job IDs {}'.format(timeout, job_ids))

        remaining_job_ids = set(job_ids)
        found_jobs = []

        countdown = timeout
        while countdown > 0:
            matched_jobs = self.find_jobs(remaining_job_ids)
            if matched_jobs:
                remaining_job_ids.difference_update({job['id'] for job in matched_jobs})
                found_jobs.extend(matched_jobs)
            if not remaining_job_ids:
                return found_jobs
            time.sleep(delay)
            countdown -= delay

        logger.error(
            'Timed out while waiting for completion of the job IDs {}. '
            'Results not updated.'.format(remaining_job_ids))

    # pylint: disable=no-self-use
    def _check_outcome(self, jobs):
        """Parses returned messages and checks submit outcome."""
        if self.skip:
            return False

        if not jobs:
            logger.error('Import failed!')
            return False

        failed_jobs = []
        for job in jobs:
            status = job.get('status')
            if not status:
                failed_jobs.append(job)
                continue

            if status.lower() != 'success':
                failed_jobs.append(job)

        for job in failed_jobs:
            logger.error('job: {0}; status: {1}'.format(job.get('id'), job.get('status')))
        if len(failed_jobs) == len(jobs):
            logger.error('Import failed!')
        elif failed_jobs:
            logger.error('Some import jobs failed!')
        else:
            logger.info('Results successfully updated!')

        return not failed_jobs

    # pylint: disable=no-self-use
    def _download_log(self, url, output_file):
        """Saves log returned by the message bus."""
        logger.info("Saving log {} to {}".format(url, output_file))

        def _do_log_download():
            try:
                return requests.get(url)
            # pylint: disable=broad-except
            except Exception as err:
                logger.error(err)

        # log file may not be ready yet, wait a bit
        for __ in range(5):
            log_data = _do_log_download()
            if log_data or log_data is None:
                break
            time.sleep(2)

        if not log_data:
            logger.error("Failed to download log file '{}'.".format(url))
            return
        with open(os.path.expanduser(output_file), 'ab') as out:
            out.write(log_data.content)

    def get_logs(self, jobs, log_file=None):
        """Get log or log url of the jobs."""
        if not jobs:
            return

        for job in jobs:
            url = job.get('logstashURL')
            if url:
                if log_file:
                    self._download_log(url, log_file)
                else:
                    logger.info('Submit log for job {0}: {1}'.format(job.get('id'), url))

    def verify_submit(self, job_ids, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY, **kwargs):
        """Verifies that the results were successfully submitted."""
        if self.skip:
            return False

        jobs = self.wait_for_jobs(job_ids, timeout, delay)
        self.get_logs(jobs, log_file=kwargs.get('log_file'))

        return self._check_outcome(jobs)


def verify_submit(
        session, queue_url, job_ids, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY, **kwargs):
    """Verifies that the results were successfully submitted."""
    verification_queue = QueueSearch(session=session, queue_url=queue_url)
    return verification_queue.verify_submit(job_ids, timeout=timeout, delay=delay, **kwargs)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,unused-argument,invalid-name

from __future__ import unicode_literals

import os

from mock import patch

from dump2polarion import verify


SEARCH_QUEUE = {
    "jobsPerPage": 50,
    "maxPages": 2,
    "currentPage": 1,
    "jobs": [{
        "id": 17977,
        "status": "SUCCESS",
        "logstashURL": "http://logstash/00017977.log"
    }, {
        "id": 17974,
        "status": "FAILED",
        "logstashURL": "http://logstash/00017974.log"
    }, {
        "id": 17976,
        "status": "SUCCESS",
        "logstashURL": "http://logstash/00017976.log"
    }]
}


class DummyResponse(object):
    def __init__(self, response):
        self.status_code = 200
        self.response = response

    def __len__(self):
        return 1

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response


# pylint: disable=too-few-public-methods
class DummySession(object):
    def __init__(self, get):
        self._get = get

    def get(self, *args, **kwargs):
        return self._get()


def download_queue_data(*args, **kwargs):
    return SEARCH_QUEUE


# pylint: disable=too-many-public-methods
class TestQueueSearch(object):

    # object init
    def test_session_missing(self, captured_log):
        vq = verify.QueueSearch(None, 'foo')
        assert 'Missing requests session, skipping' in captured_log.getvalue()
        assert vq.skip is True

    def test_url_missing(self, captured_log):
        vq = verify.QueueSearch('foo', None)
        assert 'The queue url is not configured, skipping' in captured_log.getvalue()
        assert vq.skip is True

    def test_init(self):
        vq = verify.QueueSearch('foo', 'bar')
        assert vq.skip is False

    # download queue
    def test_download_queue_none(self):
        vq = verify.QueueSearch(DummySession(lambda: DummyResponse(None)), 'bar')
        response = vq.download_queue([1, 2])
        assert response is None

    def test_download_queue_data(self):
        vq = verify.QueueSearch(DummySession(lambda: DummyResponse(SEARCH_QUEUE)), 'bar')
        response = vq.download_queue([1, 2])
        assert response == SEARCH_QUEUE

    def test_download_queue_failed(self):
        vq = verify.QueueSearch(DummySession(lambda: False), 'bar')
        response = vq.download_queue([1, 2])
        assert response is None

    def test_download_queue_exception(self, captured_log):
        def _raise():
            raise Exception('TestFail')

        vq = verify.QueueSearch(DummySession(_raise), 'bar')
        response = vq.download_queue([1, 2])
        assert response is None
        assert 'TestFail' in captured_log.getvalue()

    # find job in queue
    def test_job_not_found(self):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.find_jobs([17978, 17975])
        assert outcome == []

    def test_job_found(self):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.find_jobs([17977, 17976])
        assert outcome[0] == SEARCH_QUEUE['jobs'][0]
        assert outcome[1] == SEARCH_QUEUE['jobs'][2]

    # job log handling
    def test_get_log_failed(self, tmpdir, captured_log):
        log_file = os.path.join(str(tmpdir), 'out.log')
        jobs = [{'logstashURL': 'foo'}]
        with patch('requests.get', return_value=None):
            vq = verify.QueueSearch('foo', 'bar')
            vq.get_logs(jobs, log_file)
        assert not os.path.exists(log_file)
        assert 'Failed to download log file' in captured_log.getvalue()

    def test_get_log_saved(self, tmpdir):
        log_file = os.path.join(str(tmpdir), 'out.log')
        jobs = [{'logstashURL': 'foo'}]
        with patch('requests.get', return_value=DummyResponse(b'content')):
            vq = verify.QueueSearch('foo', 'bar')
            vq.get_logs(jobs, log_file)
        assert os.path.exists(log_file)

    def test_get_log_displayed(self, captured_log):
        jobs = [{'logstashURL': 'foo'}]
        with patch('requests.get', return_value=None):
            vq = verify.QueueSearch('foo', 'bar')
            vq.get_logs(jobs)
        assert 'Submit log for job None: foo' in captured_log.getvalue()

    def test_get_log_exception(self, tmpdir, captured_log):
        log_file = os.path.join(str(tmpdir), 'out.log')
        jobs = [{'logstashURL': 'foo'}]
        with patch('requests.get', side_effect=Exception('TestFail')):
            vq = verify.QueueSearch('foo', 'bar')
            vq.get_logs(jobs, log_file)
        assert not os.path.exists(log_file)
        assert 'TestFail' in captured_log.getvalue()

    # verify submit
    def test_queue_empty_queue(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = lambda *args: None
        outcome = vq.verify_submit([17974], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'not updated' in captured_log.getvalue()

    def test_queue_submit_failed_one(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17974, 17976, 17977], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'status: FAILED' in captured_log.getvalue()
        assert 'Some import jobs failed' in captured_log.getvalue()

    def test_queue_submit_failed_all(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17974], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'status: FAILED' in captured_log.getvalue()
        assert 'Import failed!' in captured_log.getvalue()

    def test_queue_submit_not_found_one(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17976, 17978], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'not updated' in captured_log.getvalue()
        assert 'Import failed!' in captured_log.getvalue()

    def test_queue_submit_not_found_all(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17978], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'not updated' in captured_log.getvalue()
        assert 'Import failed!' in captured_log.getvalue()

    def test_queue_submit_found(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar')
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17976, 17977], timeout=0.0000001, delay=0.0000001)
        assert outcome
        assert 'successfully updated' in captured_log.getvalue()

    def test_queue_submit_skip(self):
        vq = verify.QueueSearch('foo', 'bar')
        vq.skip = True
        outcome = vq.verify_submit([17977], timeout=0.0000001, delay=0.0000001)
        assert outcome is False



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Dump testcases results to xunit file for submitting to the Polarion XUnit Importer.
"""

from __future__ import absolute_import, unicode_literals

import datetime

from collections import namedtuple

from xml.dom import minidom
from xml.etree import ElementTree

import six

from dump2polarion import transform, utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.verdicts import Verdicts


ImportedData = namedtuple('ImportedData', 'results testrun')


class XunitExport(object):
    """Exports testcases results into Polarion xunit."""
    def __init__(self, testrun_id, tests_records, config, transform_func=None):
        self.testrun_id = testrun_id
        self.tests_records = tests_records
        self.config = config
        self._lookup_prop = ''
        self._transform_func = transform_func or transform.get_results_transform(config)

    def _top_element(self):
        """Returns top XML element."""
        top = ElementTree.Element('testsuites')
        comment = ElementTree.Comment("Generated for testrun {}".format(self.testrun_id))
        top.append(comment)
        return top

    def _properties_element(self, parent_element):
        """Returns properties XML element."""
        testsuites_properties = ElementTree.SubElement(parent_element, 'properties')

        ElementTree.SubElement(testsuites_properties, 'property',
                               {'name': 'polarion-testrun-id', 'value': str(self.testrun_id)})

        for name, value in sorted(self.config['xunit_import_properties'].items()):
            if name == 'polarion-lookup-method':
                lookup_prop = str(value).lower()
                if lookup_prop not in ('id', 'name', 'custom'):
                    raise Dump2PolarionException(
                        "Invalid value '{}' for the 'polarion-lookup-method' property".format(
                            str(value)))
                self._lookup_prop = lookup_prop
            else:
                ElementTree.SubElement(testsuites_properties, 'property',
                                       {'name': name, 'value': str(value)})

        return testsuites_properties

    def _fill_lookup_prop(self, testsuites_properties):
        """Fills the polarion-lookup-method property."""
        if not self._lookup_prop:
            raise Dump2PolarionException(
                "Failed to set the 'polarion-lookup-method' property")

        ElementTree.SubElement(testsuites_properties, 'property',
                               {'name': 'polarion-lookup-method', 'value': self._lookup_prop})

    def _testsuite_element(self, parent_element):
        """Returns testsuite XML element."""
        testsuite = ElementTree.SubElement(
            parent_element,
            'testsuite',
            {'name': 'Import for {} - {} testrun'.format(
                self.config['xunit_import_properties']['polarion-project-id'], self.testrun_id)})
        return testsuite

    @staticmethod
    def _fill_verdict(verdict, result, testcase, records):
        # xunit Pass maps to Passed in Polarion
        if verdict in Verdicts.PASS:
            records['passed'] += 1
        # xunit Failure maps to Failed in Polarion
        elif verdict in Verdicts.FAIL:
            records['failures'] += 1
            verdict_data = {'type': 'failure'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'failure', verdict_data)
        # xunit Error maps to Blocked in Polarion
        elif verdict in Verdicts.SKIP:
            records['skipped'] += 1
            verdict_data = {'type': 'error'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'error', verdict_data)
        # xunit Skipped maps to Waiting in Polarion
        elif verdict in Verdicts.WAIT:
            records['waiting'] += 1
            verdict_data = {'type': 'skipped'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'skipped', verdict_data)

    def _transform_result(self, result):
        """Calls transform function on result."""
        if self._transform_func:
            result = self._transform_func(result)
        return result or None

    @staticmethod
    # pylint: disable=inconsistent-return-statements
    def _get_verdict(result):
        """Gets verdict of the testcase."""
        verdict = result.get('verdict')
        if not verdict:
            return
        verdict = verdict.strip().lower()
        if verdict not in Verdicts.PASS + Verdicts.FAIL + Verdicts.SKIP + Verdicts.WAIT:
            return
        return verdict

    # pylint: disable=inconsistent-return-statements
    def _check_lookup_prop(self, testcase_id, testcase_title):
        """Checks that selected lookup property can be used for this testcase."""
        if self._lookup_prop:
            if not testcase_id and self._lookup_prop != 'name':
                return
            if not testcase_title and self._lookup_prop == 'name':
                return
        else:
            if testcase_id:
                self._lookup_prop = 'id'
            elif testcase_title:
                self._lookup_prop = 'name'
        return True

    @staticmethod
    def _testcase_element(parent_element, result, records, testcase_id, testcase_title):
        """Creates XML element for given testcase result and update testcases records."""
        testcase_time = float(result.get('time') or result.get('duration') or 0)
        records['time'] += testcase_time

        testcase_data = {
            'name': testcase_title or testcase_id,
            'time': str(testcase_time)}
        if result.get('classname'):
            testcase_data['classname'] = result['classname']
        testcase = ElementTree.SubElement(parent_element, 'testcase', testcase_data)
        return testcase

    @staticmethod
    def _fill_out_err(result, testcase):
        """Adds stdout and stderr if present."""
        if result.get('stdout'):
            system_out = ElementTree.SubElement(testcase, 'system-out')
            system_out.text = utils.get_unicode_str(result['stdout'])

        if result.get('stderr'):
            system_err = ElementTree.SubElement(testcase, 'system-err')
            system_err.text = utils.get_unicode_str(result['stderr'])

    @staticmethod
    def _fill_properties(verdict, result, testcase, testcase_id, testcase_title):
        """Adds properties into testcase element."""
        properties = ElementTree.SubElement(testcase, 'properties')
        ElementTree.SubElement(
            properties,
            'property',
            {'name': 'polarion-testcase-id', 'value': testcase_id or testcase_title}
        )
        if verdict in Verdicts.PASS and result.get('comment'):
            ElementTree.SubElement(
                properties,
                'property',
                {'name': 'polarion-testcase-comment',
                 'value': utils.get_unicode_str(result['comment'])}
            )

        for param, value in six.iteritems(result.get('params', {}) or {}):
            ElementTree.SubElement(
                properties,
                'property',
                {'name': 'polarion-parameter-{}'.format(param),
                 'value': utils.get_unicode_str(value)}
            )

    def _gen_testcase(self, parent_element, result, records):
        """Creates record for given testcase result."""
        result = self._transform_result(result)
        if not result:
            return

        verdict = self._get_verdict(result)
        if not verdict:
            return

        testcase_id = result.get('id')
        testcase_title = result.get('title')

        if not self._check_lookup_prop(testcase_id, testcase_title):
            return

        testcase = self._testcase_element(
            parent_element, result, records, testcase_id, testcase_title)

        self._fill_verdict(verdict, result, testcase, records)
        self._fill_out_err(result, testcase)
        self._fill_properties(verdict, result, testcase, testcase_id, testcase_title)

    def _fill_tests_results(self, testsuite_element):
        """Creates records for all testcases results."""
        if not self.tests_records.results:
            raise NothingToDoException("Nothing to export")

        records = dict(passed=0, skipped=0, failures=0, waiting=0, time=0.0)
        for testcase_result in self.tests_records.results:
            self._gen_testcase(testsuite_element, testcase_result, records)

        tests_num = (
            records['passed'] +
            records['skipped'] +
            records['failures'] +
            records['waiting'])

        if tests_num == 0:
            raise NothingToDoException("Nothing to export")

        testsuite_element.set('errors', str(records['skipped']))
        testsuite_element.set('failures', str(records['failures']))
        testsuite_element.set('skipped', str(records['waiting']))
        testsuite_element.set('time', '{0:.4f}'.format(records['time']))
        testsuite_element.set('tests', str(tests_num))

    @staticmethod
    def _prettify(top_element):
        """Returns a pretty-printed XML."""
        rough_string = ElementTree.tostring(top_element, 'utf-8')
        reparsed = minidom.parseString(rough_string)
        return utils.get_unicode_str(reparsed.toprettyxml(indent='  ', encoding='utf-8'))

    def export(self):
        """Returns xunit XML."""
        top = self._top_element()
        properties = self._properties_element(top)
        testsuite = self._testsuite_element(top)
        self._fill_tests_results(testsuite)
        self._fill_lookup_prop(properties)
        return self._prettify(top)

    def write_xml(self, xml, output_file=None):
        """Outputs the XML content into a file."""
        gen_filename = 'testrun_{}-{:%Y%m%d%H%M%S}.xml'.format(
            self.testrun_id, datetime.datetime.now())
        utils.write_xml(xml, output_loc=output_file, filename=gen_filename)
